@@ -1,5 +1,5 @@
-# Copyright (c) 2012 Mitch Garnaat http://garnaat.org/
-# Copyright 2012 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright (c) 2012-2013 Mitch Garnaat http://garnaat.org/
+# Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the
@@ -23,7 +23,7 @@
 import re
 import logging
 
-__version__ = '0.4.2'
+__version__ = '0.5.0'
 
 
 class NullHandler(logging.Handler):
@@ -39,7 +39,7 @@ _first_cap_regex = re.compile('(.)([A-Z][a-z]+)')
 _number_cap_regex = re.compile('([a-z])([0-9]+)')
 _end_cap_regex = re.compile('([a-z0-9])([A-Z])')
 
-ScalarTypes = ('string', 'integer', 'boolean', 'timestamp', 'float')
+ScalarTypes = ('string', 'integer', 'boolean', 'timestamp', 'float', 'double')
 
 
 def xform_name(name, sep='_'):
